#include "loader.hpp"
#include <iostream>
using namespace std;

extern "C" {
	#include "lua.h"
	#include "lualib.h"
	#include "lauxlib.h"
}


LevelData::LevelData(string s)
{
    loaddata(s);
}

void LevelData::loaddata(string s)
{
    lua_State* L;
    	/* initialize Lua */
	L = lua_open();

	/* load Lua base libraries */
	luaL_openlibs(L);

	/* load the script */
	luaL_dofile(L, s.c_str());

    //step 1: read cities/populations
    cities.clear();
    names.clear();
    populations.clear();

    int i=1;
    lua_getglobal(L,"cities");

    while (true)
    {

		
        Point city;
        Demographics population;

        //get the next table
        lua_pushinteger(L,i);
		lua_gettable(L, -2 );
		
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }
		
            //CITY NAME
                lua_pushinteger(L,1);
                lua_gettable(L,-2);
                names.push_back(lua_tostring(L,-1));
                lua_pop(L,1);
		
            //POSITION
                lua_pushinteger(L,2);
                lua_gettable(L,-2);
                city.x=lua_tonumber(L,-1);
                lua_pop(L,1);
		
                lua_pushinteger(L,3);
                lua_gettable(L,-2);
                city.y=lua_tonumber(L,-1);
                lua_pop(L,1);
		
            //STARTING POPULATION
                lua_pushinteger(L,4);
                lua_gettable(L,-2);
                population.a=lua_tonumber(L,-1);
                lua_pop(L,1);
		
                lua_pushinteger(L,5);
                lua_gettable(L,-2);
                population.b=lua_tonumber(L,-1);
                lua_pop(L,1);
		

            lua_pop(L,1);

        cities.push_back(city);
        populations.push_back(population);

        i++;
		
    }
    lua_pop(L,1);
	
	

    //step 2:load dialogues

    // A:INTRO

    intro.clear();

    i=1;
    lua_getglobal(L,"intro");
    while (true)
    {
        DialogueInfo di;

        //get the next table
        lua_pushinteger(L,i);
            if (lua_isnil(L,-1))//is this necessary?
            {
                lua_pop(L,1);
                break;
            }
            lua_gettable(L, -2 );
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }

                lua_pushinteger(L,1);
                lua_gettable(L,-2);
                di.speaker=lua_tointeger(L,-1);
                lua_pop(L,1);

                lua_pushinteger(L,2);
                lua_gettable(L,-2);
                di.dialogue=lua_tostring(L,-1);
                lua_pop(L,1);

            lua_pop(L,1);

        intro.push_back(di);

        i++;
    }
    lua_pop(L,1);
	

    // B:OUTRO
    outro.clear();

    i=1;
    lua_getglobal(L,"outro");
    while (true)
    {
        DialogueInfo di;

        //get the next table
        lua_pushinteger(L,i);
            lua_gettable(L, -2 );
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }

                lua_pushinteger(L,1);
                lua_gettable(L,-2);
                di.speaker=lua_tointeger(L,-1);
                lua_pop(L,1);

                lua_pushinteger(L,2);
                lua_gettable(L,-2);
                di.dialogue=lua_tostring(L,-1);
                lua_pop(L,1);

            lua_pop(L,1);

        outro.push_back(di);

        i++;
    }
    lua_pop(L,1);

    //step 3:load race profiles
    profiles.clear();
	
	

    i=1;
    lua_getglobal(L,"profiles");
    while (true)
    {
        RaceProfile profile;

        //get the next table
        lua_pushinteger(L,i);
            lua_gettable(L, -2 );
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }

                lua_pushstring(L,"aggression");
                lua_gettable(L,-2);
                profile.aggression=lua_tonumber(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"mortality");
                lua_gettable(L,-2);
                profile.mortality=lua_tonumber(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"migrationspeed");
                lua_gettable(L,-2);
                profile.migrationspeed=lua_tonumber(L,-1);
                lua_pop(L,1);

            lua_pop(L,1);

        profiles.push_back(profile);

        i++;
    }
    lua_pop(L,1);

	
    //step 4:load level end condtiions
    winconditions.clear();

    i=1;
    lua_getglobal(L,"wincondition");
    while (true)
    {
        WinCondition wc;

        //get the next table
        lua_pushinteger(L,i);
            lua_gettable(L, -2 );
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }

                lua_pushstring(L,"city");
                lua_gettable(L,-2);
                wc.city=lua_tointeger(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"people");
                lua_gettable(L,-2);
                wc.people=lua_tointeger(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"cond");
                lua_gettable(L,-2);
                wc.condition=lua_tostring(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"amt");
                lua_gettable(L,-2);
                wc.amount=lua_tonumber(L,-1);
                lua_pop(L,1);

            lua_pop(L,1);

        winconditions.push_back(wc);

        i++;
    }
    lua_pop(L,1);

//LOAD FAMINES

    famines.clear();

 i=1;
    lua_getglobal(L,"famines");
    while (true)
    {
        Famine f;

        //get the next table
        lua_pushinteger(L,i);
            lua_gettable(L, -2 );
            if (lua_isnil(L,-1))
            {
                lua_pop(L,1);
                break;
            }

                lua_pushstring(L,"city");
                lua_gettable(L,-2);
                f.city=lua_tointeger(L,-1)-1;
                lua_pop(L,1);

                lua_pushstring(L,"start");
                lua_gettable(L,-2);
                f.a=lua_tonumber(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"stop");
                lua_gettable(L,-2);
                f.b=lua_tonumber(L,-1);
                lua_pop(L,1);

                lua_pushstring(L,"rate");
                lua_gettable(L,-2);
                f.rate=lua_tonumber(L,-1);
                lua_pop(L,1);

            lua_pop(L,1);

        famines.push_back(f);

        i++;
    }
    lua_pop(L,1);
	
	

//LOAD THE ARROW TYPE SETTINGS

    lua_getglobal(L,"arrowtypes");

        lua_pushstring(L,"both");
        lua_gettable(L,-2);
        enabledarrows.both=lua_toboolean(L,-1);
        lua_pop(L,1);

        lua_pushstring(L,"player");
        lua_gettable(L,-2);
        enabledarrows.player=lua_toboolean(L,-1);
        lua_pop(L,1);

        lua_pushstring(L,"enemy");
        lua_gettable(L,-2);
        enabledarrows.enemy=lua_toboolean(L,-1);
        lua_pop(L,1);


    lua_pop(L,1);


    lua_close(L);
	

}

