#ifndef MAINPART_HPP
#define MAINPART_HPP

#include <vector>
#include <set>
#include <stack>
#include "boost/multi_array.hpp"
using namespace std;
using namespace boost;


#ifdef __APPLE__
#include <SDL/SDL.h>
#else
#include <SDL.h>
#endif


#include "Engine/engine.hpp"
#include "globals.hpp"
#include "entities.hpp"
#include "widget.hpp"
#include "math.hpp"


const int PLAY_WIDTH=640;
const int PLAY_HEIGHT=364;

const float PLAYBACKTIME=10;

enum Arrow_Type {at_both=0,at_player=1,at_enemy=2, at_remove=3};

Arrow_Type inttoatype(int a);

class Arrow
{
public:
    unsigned int a;
    unsigned int b;
    Arrow_Type  atype;
    const bool operator == (const Arrow& a2) const
    {
        return (a==a2.a)&&(b==a2.b);
    };
    Arrow(unsigned int e_a, unsigned int e_b, Arrow_Type e_atype):a(e_a),b(e_b),atype(e_atype){};
    Arrow(unsigned int e_a, unsigned int e_b, int e_atype):a(e_a),b(e_b),atype(inttoatype(e_atype)){};
};

bool overlap(const Arrow& a, const Arrow&b);
bool same(const Arrow& a, const Arrow&b);

class Time
{
public:
    GLfloat t;
    const bool operator < (const Time& t2) const
    {
        return t<t2.t;
    };
    Time(const Time& time):
            t(time.t){};
    Time(GLfloat e_t):
            t(e_t){};
};

class Event: public Time, public Arrow
{
public:
    const bool operator == (const Event& t2) const
    {
        return (t==t2.t)&&
               (atype==t2.atype)&&
               (a==t2.a)&&
               (b==t2.b);
    };
    const bool operator < (const Event& t2) const
    {
        if (t==t2.t)//these layers are pretty arbitrary; just wanted to fix some ordering
        {
            //next order by city
            unsigned int a1, b1,a2,b2;//normalize the form so that a1<=b1:
            //this makes it symmetrical
            if (a<b)
            {
                a1=a;
                b1=b;
            }
            else
            {
                a1=b;
                b1=a;
            }

            if (t2.a<t2.b)
            {
                a2=t2.a;
                b2=t2.b;
            }
            else
            {
                a2=t2.b;
                b2=t2.a;
            }

            if (a1!=a2)
            {
                return a1<a2;
            }
            else if (b1!=b2)
            {
                return b1<b2;
            }
            else
            {
                // return atype<t2.atype; < could comment this out if wanted to distinguish direction
                return false;
            }

            /*
            if (atype==t2.atype)
            {
                if (a==t2.a)
                    return (b<t2.b);
                else
                    return (a<t2.a);
            }
            else return atype<t2.atype;*/
        }
        else
            return t<t2.t;
    };
    Event(GLfloat e_time, int e_a, int e_b,Arrow_Type e_atype):
            Time(e_time),
            Arrow(e_a,e_b,e_atype)
    {};
    Event(GLfloat e_time, int e_a, int e_b,int e_atype):
            Time(e_time),
            Arrow(e_a,e_b,e_atype)
    {};

};

class FamineData {
public:
  unsigned int city;
  float rate;
  FamineData(int c, float r):city(c), rate(r){};
};

const int SLICECOUNT=200;


struct Slice
{
    vector<Arrow> arrows;
    vector<Demographics > populations;
    Demographics totalpops;
    vector<FamineData> famines;
};

class MainPart: public EnginePart
{
public:
    void reset();
    MainPart(SubengineInterface* subengines, Global_Variables* p_global_variables);
    ~MainPart();
protected:
    int input(const SDL_Event& event);
    int logic();
    void draw();
private:
    int submitparam;
    int genericparam;
    int arrowmode;
    int showmenu;
    int newarrowmode;
    int menuparam;
    Screen_Element* capturing;
    vector<Screen_Element*> widgets;
    int focus;//gui element focus
    int citycount;
    void calcSlices();
    vector<Slice*> slices;
    void populate();
    vector<Point> points;
    multi_array<float, 2> distancesq;
    vector<Demographics > populations;
    Demographics totalpops;
    vector<Demographics > startingpopulations;
    vector<Arrow> arrows;
    vector<FamineData> famines;
    set<Event> events;
    void calcArrows(vector<Arrow>& arrs, float t);
    void calcFamines(vector<FamineData>& fams, float t);
    void highlightCity();
    void highlightArrow();
    void setTime(GLfloat d);
    const int arrowtypeat(Arrow a,const vector<Arrow>& arrs ) const;
    const bool isAlreadyOff(const Arrow& a) const;
    const bool isAlreadyValue(const Arrow& a) const;
    void deleteArrowEvent(const Arrow& a, GLfloat t);
    void updatearrowindices();
    const bool startedAt(const Arrow& a) const;
    const int atype_before(const Arrow& a) const;
    void deleteEnd(const Arrow& a);
    void deleteEndAfter(const Arrow& a)    ;
    bool tryDelete(int a, int b);
    void deleteRepetitionsAfter(const Arrow& a);
    void calcHighlightInterval();
    const bool isAlreadyOn(const Arrow& a) const;
    const GLfloat getTime() const;
    const bool finished() const;
    void playInput();
    void setMenu();
    GLfloat time;
    enum MouseModes {mm_default, mm_dragpoint, mm_dragline} mousemode;
    enum LevelMode {lm_intro, lm_game, lm_menu,lm_playback,lm_outro} levelmode;
    int highlighted;
    int highlightedarrow;
    unsigned int cutscenemarker;
    float highlightedarrowbegin;
    float highlightedarrowend;
    bool dragmode;
    int draglineselected;
    int dragsource;
    bool previouslyreleased;
    string tooltip;
    Global_Variables* global_variables;
    int doundo;
    void backupEvents();
    void undo();
    stack <set<Event> > backups;
    int restartparam;
};

#endif /* MAINPART_HPP */
